@@ -21,6 +21,7 @@
 #     https://www.nipreps.org/community/licensing/
 #
 """Utilities."""
+from itertools import product
 from nipype.interfaces.base import (
     BaseInterfaceInputSpec,
     TraitedSpec,
@@ -37,6 +38,8 @@ from nipype.interfaces.ants.segmentation import (
 )
 from nipype.interfaces.mixins import CopyHeaderInterface as _CopyHeaderInterface
 
+from sdcflows.utils.tools import reorient_pedir
+
 OBLIQUE_THRESHOLD_DEG = 0.5
 
 
@@ -147,6 +150,75 @@ class UniformGrid(SimpleInterface):
         return runtime
 
 
+class _ReorientImageAndMetadataInputSpec(TraitedSpec):
+    in_file = File(exists=True, mandatory=True, desc="Input 3- or 4D image")
+    target_orientation = traits.Str(
+        desc="Axis codes of coordinate system to reorient to"
+    )
+    pe_dir = InputMultiObject(
+        traits.Enum(
+            *["".join(p) for p in product("ijkxyz", ("", "-"))],
+            mandatory=True,
+            desc="Phase encoding direction",
+        )
+    )
+
+
+class _ReorientImageAndMetadataOutputSpec(TraitedSpec):
+    out_file = File(desc="Reoriented image")
+    pe_dir = OutputMultiObject(
+        traits.Enum(
+            *["".join(p) for p in product("ijkxyz", ("", "-"))],
+            desc="Phase encoding direction in reoriented image",
+        )
+    )
+
+
+class ReorientImageAndMetadata(SimpleInterface):
+    input_spec = _ReorientImageAndMetadataInputSpec
+    output_spec = _ReorientImageAndMetadataOutputSpec
+
+    def _run_interface(self, runtime):
+        import numpy as np
+        import nibabel as nb
+        from nipype.utils.filemanip import fname_presuffix
+
+        target = self.inputs.target_orientation.upper()
+        if not all(code in "RASLPI" for code in target):
+            raise ValueError(
+                f"Invalid orientation code {self.inputs.target_orientation}"
+            )
+
+        img = nb.load(self.inputs.in_file)
+        img2target = nb.orientations.ornt_transform(
+            nb.io_orientation(img.affine),
+            nb.orientations.axcodes2ornt(self.inputs.target_orientation),
+        ).astype(int)
+
+        # Identity transform
+        if np.array_equal(img2target, [[0, 1], [1, 1], [2, 1]]):
+            self._results = dict(
+                out_file=self.inputs.in_file,
+                pe_dir=self.inputs.pe_dir,
+            )
+            return runtime
+
+        reoriented = img.as_reoriented(img2target)
+
+        pe_dirs = [reorient_pedir(pe_dir, img2target) for pe_dir in self.inputs.pe_dir]
+
+        self._results = dict(
+            out_file=fname_presuffix(
+                self.inputs.in_file, suffix=target, newpath=runtime.cwd
+            ),
+            pe_dir=pe_dirs,
+        )
+
+        reoriented.to_filename(self._results["out_file"])
+
+        return runtime
+
+
 class _ConvertWarpInputSpec(BaseInterfaceInputSpec):
     in_file = File(exists=True, mandatory=True, desc="output of 3dQwarp")
 
@@ -381,7 +453,7 @@ def _deoblique(in_file, in_affine=None, newpath=None):
     from nipype.utils.filemanip import fname_presuffix
 
     nii = nb.load(in_file)
-    if np.all(nb.affines.obliquity(nii.affine)) < OBLIQUE_THRESHOLD_DEG:
+    if np.all(nb.affines.obliquity(nii.affine) < OBLIQUE_THRESHOLD_DEG):
         return in_file
 
     if in_affine is None:
@@ -414,7 +486,7 @@ def _reoblique(in_epi, in_plumb, in_field, in_mask=None, newpath=None):
     from nipype.utils.filemanip import fname_presuffix
 
     epinii = nb.load(in_epi)
-    if np.all(nb.affines.obliquity(epinii.affine)) < OBLIQUE_THRESHOLD_DEG:
+    if np.all(nb.affines.obliquity(epinii.affine) < OBLIQUE_THRESHOLD_DEG):
         return in_plumb, in_field, in_mask
 
     out_files = [
